/* acre - A cairo-based library for creating plots and charts.
 *
 * Copyright © 2009 Carl Worth <cworth@cworth.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */

#ifndef ACRE_H
#define ACRE_H

#include <pango/pangocairo.h>

typedef struct _acre acre_t;
typedef struct _acre_data acre_data_t;

/* Create a new, empty plot. */
acre_t *
acre_create (void);

/* Destroy a plot. */
void
acre_destroy (acre_t *acre);

void
acre_set_title (acre_t *acre, const char *title);

void
acre_set_x_axis_label (acre_t *acre, const char *label);

void
acre_set_y_axis_label (acre_t *acre, const char *label);

/* Get the range of data values in the X dimension. */
void
acre_get_x_axis_data_range (acre_t *acre, double *x_min, double *x_max);

/* Get the range of the currently viewable X axis. */
void
acre_get_x_axis_range (acre_t *acre, double *x_min, double *x_max);

/* Set the range of the currently viewable X axis. */
void
acre_set_x_axis_range (acre_t *acre, double x_min, double x_max);

/* Set the X axis viewable range to be sized automatically.
 *
 * If both X and Y axes are sized automatically, then the X range will
 * be set to the full range of data in the X dimension.
 *
 * But if the Y axis has been set to a limited viewable range, (with
 * acre_set_y_axis_range), then the X axis will be set to the
 * corresponding X range of data that lies within that set Y range. */
void
acre_set_x_axis_range_auto (acre_t *acre);

/* Get the range of data values in the Y dimnension. */
void
acre_get_y_axis_data_range (acre_t *acre, double *y_min, double *y_max);

/* Get the range of the currently viewable Y axis. */
void
acre_get_y_axis_range (acre_t *acre, double *y_min, double *y_max);

/* Set the range of the currently viewable Y axis. */
void
acre_set_y_axis_range (acre_t *acre, double y_min, double y_max);

/* Set the Y axis viewable range to be sized automatically.
 *
 * If both X and Y axes are sized automatically, then the Y range will
 * be set to the full range of data in the Y dimension.
 *
 * But if the X axis has been set to a limited viewable range, (with
 * acre_set_x_axis_range), then the Y axis will be set to the
 * corresponding Y range of data that lies within that set X range. */
void
acre_set_y_axis_range_auto (acre_t *acre);

/* Add a dataset to the plot. The plot assumes ownership of the
 * dataset so it is not necessary to call acre_data_destroy on it.
 */
void
acre_add_data (acre_t *acre, acre_data_t *data);

/* Draw the plot to the given cairo context within a user-space
 * rectangle from (0, 0) to (width, height). This size includes all
 * space for extra-plot elements (such as the title, the axis labels,
 * etc.)
 */
void
acre_draw (acre_t *acre, cairo_t *cr, int width, int height);

/* Create a new dataset---a collection of (x, y) datapoints. A single
 * plot can contain multiple datasets, (see acre_add_data). */
acre_data_t *
acre_data_create (void);

/* Destroy an acre dataset. Do not call this function if the dataset
 * has been added to an acre_t plot with acre_add_data. */
void
acre_data_destroy (acre_data_t *data);

/* The style to be used for rendering data sets. */
typedef enum
{
	/* A simple line graph connection each (X,Y) pair in order. */
	ACRE_STYLE_LINE,

	/* A bar chart. */
	ACRE_STYLE_BARS,

	/* Bars if there is room for them, a line otherwise. */
	ACRE_STYLE_BARS_OR_LINE,

	/* A timeline.
	 *
	 * Each (X,Y) pair specifies the (start,stop) time of a
	 * timeline bar.
	 *
	 * The acre_data_add_point_2d_named function is particularly
	 * useful to provide a name for each timeline bar.
	 */
	ACRE_STYLE_TIMELINE
} acre_style_t;

/* Set the rendering style for this dataset. */
void
acre_data_set_style (acre_data_t *data, acre_style_t style);

/* Set the label for this dataset (to appear in the plot's key). */
void
acre_data_set_name (acre_data_t *data, const char *name);

/* Add a datapoint to the given dataset. */
void
acre_data_add_point_2d (acre_data_t *data, double x, double y);

/* Add a datapoint with a name to the given dataset. */
void
acre_data_add_point_2d_named (acre_data_t *data, double x, double y,
			      const char *name);

#endif /* ACRE_H */
